package edu.sdsc.matrix.srb.core;

import javax.xml.bind.*;
import edu.sdsc.matrix.srb.parser.*;

public class JaxbUtil {
    
    private static ObjectFactory objFactory = new ObjectFactory();
    private static final String FAULTHANDLERS_NAME = "faulthandler";
    private static final String DOC_AUTHOR = "Erik Vandekieft";
    private static final String DOC_VERSION = "1.0";
    private static final String DOC_DOCUMENTATION = "M A T R I X: Still under development";
    private static final String DEFAULT_ID = "demomaker";
    private static final String DEFAULT_PASSWORD = "QQQQQ";
    private static final String DEFAULT_ORGANIZATION = "ucsd";
    private static final String DEFAULT_HOMEDIRECTORY = "/home/demomaker.sdsc";
    private static final String DEFAULT_DSR = "ora-sdsc";
    private static final String DEFAULT_SERVER = "srb.sdsc.edu";
    private static final int DEFAULT_PORT = 5615;

    public static DocMeta createDocMeta() 
	throws JAXBException {
	DocMeta docMeta = objFactory.createDocMeta();
	docMeta.setDocAuthor(DOC_AUTHOR);
	docMeta.setDocOwner(createUserInfo());
	docMeta.setDocVersion(DOC_VERSION);
	docMeta.setDocumentation(DOC_DOCUMENTATION);
	return docMeta;
    }

    public static UserInfo createUserInfo() 
	throws JAXBException {
	UserInfo ui = objFactory.createUserInfo();
	ui.setUserID(DEFAULT_ID);
	ui.setChallengeResponse(DEFAULT_PASSWORD); // my userid is the same as my password... for now
	ui.setHomeDirectory(DEFAULT_HOMEDIRECTORY);
	ui.setDefaultStorageResource(DEFAULT_DSR);
	ui.setOrganization(createOrganizationInfo());
	return ui;
    }
    
    public static UserInfo createUserInfo(String username, 
					  String password, 
					  String homeDirectory, 
					  String defaultStorageResource,
					  String organization)
	throws JAXBException {

	UserInfo ui = objFactory.createUserInfo();
	ui.setUserID(username);
	ui.setChallengeResponse(password);
	ui.setHomeDirectory(homeDirectory);
	ui.setDefaultStorageResource(defaultStorageResource);
	ui.setOrganization(createOrganizationInfo(organization));
	return ui;
    }

    public static OrganizationInfo createOrganizationInfo() 
	throws JAXBException {
	OrganizationInfo oi = objFactory.createOrganizationInfo();
	oi.setOrganizationName(DEFAULT_ORGANIZATION);
	return oi;
    }

    public static OrganizationInfo createOrganizationInfo(String organization) 
	throws JAXBException {
	OrganizationInfo oi = objFactory.createOrganizationInfo();
	oi.setOrganizationName(organization);
	return oi;
    }

    public static VirtualOrganizationInfo createVirtualOrganizationInfo() 
	throws JAXBException {
	VirtualOrganizationInfo voi = objFactory.createVirtualOrganizationInfo();
	voi.setServer(DEFAULT_SERVER);
	voi.setPort(DEFAULT_PORT);
	return voi;
    }

    public static VirtualOrganizationInfo createVirtualOrganizationInfo(String server, int port)
	throws JAXBException {

	VirtualOrganizationInfo voi = 
	    objFactory.createVirtualOrganizationInfo();
	voi.setServer(server);
	voi.setPort(port);
	return voi;
    }

    public static FaultHandlers createFaultHandlers() 
	throws JAXBException {
	FaultHandlers fh = objFactory.createFaultHandlers();
	fh.setName(FAULTHANDLERS_NAME);
	fh.setTBA("TBA");
	return fh;
    }

    public static StepStatusResponse createErrorStepStatusResponse(String stepID)
	throws JAXBException {
	StepStatusResponse ssr = objFactory.createStepStatusResponse();
	ssr.setStepID(stepID);
	ssr.setStatusCode(MatrixCodes.STEP_NOT_FOUND);
	return ssr;
    }
    
	public static Operation createRenameOpForCollection(String collName,String newName)
	throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		RenameOp renameOp	 = 	 objFactory.createRenameOp();
		RenameParams renameParams = objFactory.createRenameParams();

		StdDatagridObject stdDatagridObject = createStdDatagridObjectForCollection(collName);
		renameParams.setOldName(stdDatagridObject);
		renameParams.setNewName(newName);
		renameOp.setStdParams(renameParams);
		operation.setRenameOp(renameOp);

		return(operation);
	}
	
	public static Operation createRenameOpForDataSet(String dataSetName,String logicalLocation,String newName)
		throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		RenameOp renameOp	 = 	 objFactory.createRenameOp();
		RenameParams renameParams = objFactory.createRenameParams();

		StdDatagridObject stdDatagridObject = createStdDatagridObjectForDataSet(dataSetName,logicalLocation);
		renameParams.setOldName(stdDatagridObject);
		renameParams.setNewName(newName);
		renameOp.setStdParams(renameParams);
		operation.setRenameOp(renameOp);

		return(operation);
	}
    
	public static Operation createDeleteOpForCollection(String collName)
	throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		DeleteOp deleteOp	 = 	 objFactory.createDeleteOp();
		DeleteParams deleteParams = objFactory.createDeleteParams();
	
		StdDatagridObject stdDatagridObject = createStdDatagridObjectForCollection(collName);
		deleteParams.setGridObject(stdDatagridObject);
		deleteOp.setStdParams(deleteParams);
		operation.setDeleteOp(deleteOp);
	
		return(operation);
	}
	
	public static Operation createDeleteOpForDataSet(String dataIdentifier,String logicalLocation)
	throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		DeleteOp deleteOp	 = 	 objFactory.createDeleteOp();
		DeleteParams deleteParams = objFactory.createDeleteParams();

		StdDatagridObject stdDatagridObject = createStdDatagridObjectForDataSet(dataIdentifier,logicalLocation);
		deleteParams.setGridObject(stdDatagridObject);
		deleteOp.setStdParams(deleteParams);
		operation.setDeleteOp(deleteOp);

		return(operation);
	}
	
    
	public static Operation createCreateOpForContainer(String contName)
	throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		CreateOp createOp	 = 	 objFactory.createCreateOp();
		CreateParams createParams = objFactory.createCreateParams();
		
		StdDatagridObject stdDatagridObject = createStdDatagridObjectForContainer(contName);
		createParams.setDatagridObject(stdDatagridObject);
		createOp.setStdParams(createParams);
		operation.setCreateOp(createOp);
		
		return(operation);
	}
	
	public static Operation createCreateOpForCollection(String collName)
	throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		CreateOp createOp	 = 	 objFactory.createCreateOp();
		CreateParams createParams = objFactory.createCreateParams();
	
		StdDatagridObject stdDatagridObject = createStdDatagridObjectForCollection(collName);
		createParams.setDatagridObject(stdDatagridObject);
		createOp.setStdParams(createParams);
		operation.setCreateOp(createOp);
	
		return(operation);
	}
	
	public static StdDatagridObject createStdDatagridObjectForContainer(String containerName)
	throws JAXBException
	{
		Container container = 	objFactory.createContainer();
		StdDatagridObject stdDatagridObject = objFactory.createStdDatagridObject();
		container.setContainerName(containerName);
		stdDatagridObject.setContainerNow(container);
		return(stdDatagridObject);
	}
	
	public static StdDatagridObject createStdDatagridObjectForCollection(String collectionName)
	throws JAXBException
	{
		Collection collection = 	objFactory.createCollection();
		Collection.StdCollectionType stdCollectionType = objFactory.createCollectionStdCollectionType();
		StdDatagridObject stdDatagridObject = objFactory.createStdDatagridObject();
		
		stdCollectionType.setCollectionName(collectionName);
		collection.setStdCollection(stdCollectionType);
		stdDatagridObject.setCollectionNow(collection);
		
		return(stdDatagridObject);
	}
	
	public static StdDatagridObject createStdDatagridObjectForDataSet(String dataIdentifier,String logicalLocation)
	throws JAXBException
	{
		StdDatagridObject stdDatagridObject = objFactory.createStdDatagridObject();	
		DataSet dataSet = objFactory.createDataSet();
		
		dataSet.setDataIdentifier(dataIdentifier);
		dataSet.setLogicalLocation(logicalLocation);
		stdDatagridObject.setDataSetNow(dataSet);
	
		return(stdDatagridObject);
	}
	
	public static Operation createIngestOp(String dataIdentifier,String locigalLocation,String contentId)
		throws JAXBException
	{	
		Operation operation  = 	objFactory.createOperation();
		IngestDataSetOp ingestOp = 	 objFactory.createIngestDataSetOp();
		IngestDataSetParams ingestParams = objFactory.createIngestDataSetParams();
		DataSet dataSet = objFactory.createDataSet();
		
		dataSet.setDataIdentifier(dataIdentifier);
		dataSet.setLogicalLocation(locigalLocation);
		ingestParams.setDataDestination(dataSet);
		ingestParams.setContentMIMEID(contentId);
			
		ingestOp.setStdParams(ingestParams);
		operation.setIngestOp(ingestOp);		
		return(operation);
	}
}
