/* ====================================================================
 * SDSC Matrix Software License, Version 1.0
 *
 * Copyright (c) 2003 San Diego Supercomputer Center (SDSC), San Diego, USA.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that ALL the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        SDSC Matrix Project (http://www.npaci.edu/DICE/SRB/matrix/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SDSC" and "SDSC Matrix" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact arun@sdsc.edu or srb-chat@sdsc.edu.
 *
 * 5. Products derived from this software may not be called "SDSC",
 *    nor may "SDSC" appear in their name, without prior written
 *    permission of the SDSC.
 *
 * 6. License is not granted for commercial resale, in whole or in part, 
 *	  without the prior written permission from SDSC.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SDSC OR
 * ITS CONTRIBUTORS (SDSC Matrix Team) BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software is intended to consist of voluntary contributions made by many
 * individuals for the SDSC Matrix Project.
 *
 * SDSC Matrix.  
 * Designer & Architect: Arun swaran Jagatheesan (arun@sdsc.edu)
 *
 * Authors (Credits 02/SEP/2003):
 * Allen Ding (alding@sdsc.edu)
 * Arun Jagatheesan (arun@sdsc.edu)
 * Reena Mathew	(rmathew@sdsc.edu)
 * Erik Vandekieft (evk@sdsc.edu)
 */



package edu.sdsc.matrix.srb.core;

/**
 * Class represents the Connection for Matrix Server Application
 * Matrix Connection is represented using the following:<br>
 * - Matrix Account<br>
 * - Grid Connection <br>
 * - Ticket ID <br>
 * - Grid Type (Currently on SRB)<br> 
 * @author rmathew
 *
 *
 */
public class MatrixConnection
{
	/**
	 * Matrix Account
	 */
	private MatrixAccount matrixAccount;
	
	/**
	 * Grid Connection
	 */
	private GridConnection gridConnection;
	
	/**
	 * Ticket Id
	 */
	private String ticketId;
	
	/**
	 * Grid Type
	 */
	private String gridType;	
	
	/**
	 * Public Constructor
	 */
	public MatrixConnection()
	{
			
	}
	
	/**
	 * Public Constructor
	 * @param matrixAcct Matrix Account
	 * @param gridConn Grid Connection
	 * @param tId Ticket Id
	 */
	public MatrixConnection(MatrixAccount matrixAcct,GridConnection gridConn,String tId)
	{
		matrixAccount = matrixAcct;
		gridConnection =gridConn;
		ticketId = tId;
	}
	
	/**
	 * Set Matrix Account
	 * @param mAcct Matrix Account
	 */
	public void setMatrixAccount(MatrixAccount mAcct)
	{
		matrixAccount = mAcct;
	}

	/**
	 * Retrieve Matrix Account
	 * @return Matrix Account
	 */
	public MatrixAccount getMatrixAccount()
	{
		return(matrixAccount);
	}	

	/**
	 * Set Grid Connection
	 * @param conn Grid Connection
	 */
	public void setGridConnection(GridConnection conn)
	{
		gridConnection = conn;
	}	

	/**
	 * Retrieve Grid Connection
	 * @return Grid Connection
	 */
	public GridConnection getGridConnection()
	{
		return(gridConnection);
	}

	/**
	 * Return an active Grid Connection<br>
	 * If the current connection is inactive, recreate a new connection using 
	 * the Matrix Account. 
	 * @return Active Grid Connection
	 */
	public GridConnection getActiveGridConnection()
	{
		GridConnection gridConn = getGridConnection();
		// Check if Connection Active
		if(gridConn!=null)
			return(gridConn);
		else
		{
			gridConn = GridConnectionFactory.createConnection(getGridType(),getMatrixAccount().getUserInfo(),getMatrixAccount().getVirtualOrganizationInfo());
		}
		setGridConnection(gridConn);
		return(gridConn);
		
	}
	
	/**
	 * Set Ticket Id
	 * @param tid Ticket Id
	 */
	public void setTicketId(String tid)
	{
		ticketId = tid;
	}	

	/**
	 * Return Ticket Id for connection
	 * @return Ticket Id for connection
	 */
	public String getTicketId()
	{
		return(ticketId);
	}
	
	/**
	 * Set Grid Type
	 * @param type Grid Type Ex. SRB
	 */
	public void setGridType(String type)
	{
		gridType = type;
	}	

	/**
	 * Return Grid Type
	 * @return Grid Type
	 */
	public String getGridType()
	{
		return(gridType);
	}
}	