/* ====================================================================
 * SDSC Matrix Software License, Version 1.0
 *
 * Copyright (c) 2003 San Diego Supercomputer Center (SDSC), San Diego, USA.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that ALL the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        SDSC Matrix Project (http://www.npaci.edu/DICE/SRB/matrix/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SDSC" and "SDSC Matrix" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact arun@sdsc.edu or srb-chat@sdsc.edu.
 *
 * 5. Products derived from this software may not be called "SDSC",
 *    nor may "SDSC" appear in their name, without prior written
 *    permission of the SDSC.
 *
 * 6. License is not granted for commercial resale, in whole or in part, 
 *	  without the prior written permission from SDSC.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SDSC OR
 * ITS CONTRIBUTORS (SDSC Matrix Team) BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software is intended to consist of voluntary contributions made by many
 * individuals for the SDSC Matrix Project.
 *
 * SDSC Matrix.  
 * Designer & Architect: Arun swaran Jagatheesan (arun@sdsc.edu)
 *
 * Authors (Credits 02/SEP/2003):
 * Allen Ding (alding@sdsc.edu)
 * Arun Jagatheesan (arun@sdsc.edu)
 * Reena Mathew	(rmathew@sdsc.edu)
 * Erik Vandekieft (evk@sdsc.edu)
 */



package edu.sdsc.matrix.srb.core;

import edu.sdsc.grid.io.srb.SRBAccount;
import edu.sdsc.grid.io.srb.SRBFileSystem;
import edu.sdsc.grid.io.srb.SRBFile;
import edu.sdsc.matrix.srb.parser.UserInfo;
import edu.sdsc.matrix.srb.parser.VirtualOrganizationInfo;
import edu.sdsc.matrix.srb.util.MatrixUtil;
import edu.sdsc.matrix.srb.util.JaxbUtil;


import edu.sdsc.grid.io.srb.SRBContainer;
import edu.sdsc.grid.io.srb.SRBException;
import edu.sdsc.grid.io.srb.SRBRandomAccessFile;
import edu.sdsc.grid.io.local.LocalFile;

import java.io.*;
import java.net.URI;

/**
 * Class extends the Grid Connection and represents a connection 
 * to SRB. It has to implement all the abstract methods of GridConnection.
 * @author rmathew
 *
 */
public class SRBConnection extends GridConnection
{
	/**
	 * SRB account used to create SRB Connection
	 */
    private SRBAccount 	srbAccount;
    
    /**
     * SRB File System used in SRB Connection
     */
    private SRBFileSystem srbFileSystem;
    
    /**
     * Public Constuctor
     */
    public SRBConnection()
    {
    	
    }
    
    /** 
     * Set SRB Account 
     * @param account SRB Account
     */
    public void setSRBAccount(SRBAccount account)
    {
    	srbAccount = account;
    }
    
    /**
     * Return SRB Account
     * @return SRB Account
     */
    public SRBAccount getSRBAccount()
    {
    	return(srbAccount);
    }
    
    /**
     * Set SRB File System
     * @param fileSys SRB File System
     */
    public void setSRBFileSystem(SRBFileSystem fileSys)
    {
    	srbFileSystem = fileSys;
    }
    
    /**
     * Return SRB File System
     * @return SRB File System
     */
    public SRBFileSystem getSRBFileSystem()
    {
    	return(srbFileSystem);
    }

	/**
	 * Returns Port Number used for making connection
	 * @return Port Number of Connection
	 */
    public String getPort()	
    {
	String port = null;
	port = String.valueOf(srbAccount.getPort());
	return(port);
    }

	/**
	 * Returns Server Name used for making connection
	 * @return Server Name for Connection
	 */
    public String getServer()
   {
	String server = null;
	server = srbAccount.getHost();
	return(server);
   }
   
   /**
    * Clears the Connection
    */
   public void clearConnection()
   {
   		// To close the SRB Connection
   		try
   		{
			MatrixUtil.toLog("SRBConnection-clearConnection()-BEGIN", MatrixUtil.DEBUG_LOG);
   			srbFileSystem.close();
			MatrixUtil.toLog("SRBConnection-clearConnection()-CONNECTION CLOSED", MatrixUtil.DEBUG_LOG);
			srbAccount = null;
			srbFileSystem = null;
			MatrixUtil.toLog("SRBConnection-clearConnection()-END", MatrixUtil.DEBUG_LOG);
   		}
   		catch(Exception e)
   		{
			MatrixUtil.toLog("SRBConnection-clearConnection-EXCEPTION:"+e, MatrixUtil.ERROR_LOG);
   		}
   }

	/**
	 * Returns an Active Connection. If the connection is inactive, the
	 * connection is recreated using the SRB File System
	 * @return SRB File System
	 */
	public SRBFileSystem getActiveConnection()
	{
		SRBFileSystem fileSysConn = null;
		try
		{
			if(getSRBFileSystem()!=null)
			{
				fileSysConn = getSRBFileSystem();
			}
			else // Recreate Connection
			{
				// Check if ticket valid
				// TO DO .......
				fileSysConn= new SRBFileSystem(srbAccount);
				setSRBFileSystem(fileSysConn);
			}
			
		}
		catch(Exception e)
		{
				
		}
		return(fileSysConn);
	}

	/**
	 * Create SRB Connection
	 * @param srbUser SRB USer Information
	 * @param virtualOrg SRB Virtual Organization Information
	 */
	public void createSRBConnection(UserInfo srbUser,VirtualOrganizationInfo virtualOrg)
	{
		try
		{
		
			MatrixUtil.toLog("SRBConnection-createSRBConnection-BEGIN", MatrixUtil.DEBUG_LOG);
			// Extract Parameters
			SRBAccount account =null;
			SRBFileSystem srbConn = null;
			String host = virtualOrg.getServer(); 
			int port = virtualOrg.getPort();
			String userName = srbUser.getUserID(); 
			String password = srbUser.getChallengeResponse();
			String homeDirectory = srbUser.getHomeDirectory();
			String mdasDomainName =  srbUser.getOrganization().getOrganizationName();
			String defaultStorageResource= srbUser.getDefaultStorageResource();

			MatrixUtil.toLog("SRBConnection-Before -SRBAccount", MatrixUtil.DEBUG_LOG);	
			// Create SRB Account
			account = new SRBAccount(host, port, userName, password,homeDirectory, mdasDomainName, defaultStorageResource );
			MatrixUtil.toLog("SRBConnection-After -host:"+host, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -port:"+port, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -userName:"+userName, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -password:"+password, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -homeDirectory:"+homeDirectory, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -mdasDomainName:"+mdasDomainName, MatrixUtil.DEBUG_LOG);
			MatrixUtil.toLog("SRBConnection-After -defaultStorageResource:"+defaultStorageResource, MatrixUtil.DEBUG_LOG);
			
			MatrixUtil.toLog("SRBConnection-After -SRBAccount",MatrixUtil.DEBUG_LOG);

			// Create SRBConnection
			MatrixUtil.toLog("SRBConnection-Before -SRBFileSystem:srbConn:"+srbConn, MatrixUtil.DEBUG_LOG);
			
			String jargonArgs[] = {host,String.valueOf(port),userName,password,homeDirectory,mdasDomainName,defaultStorageResource};
			//SRBConnection.main(jargonArgs);
			
			
			srbConn = new SRBFileSystem(account);
			
			
			
			MatrixUtil.toLog("SRBConnection-After -SRBFileSystem", MatrixUtil.DEBUG_LOG);

			// Set SRB Account and Connection
			setSRBAccount(account);
			setSRBFileSystem(srbConn);
			MatrixUtil.toLog("SRBConnection-createSRBConnection-END", MatrixUtil.DEBUG_LOG);
		}
		catch(NullPointerException npe)
		{
			MatrixUtil.toLog("SRBConnection-NullPointerException-END"+npe, MatrixUtil.ERROR_LOG);	
		}
		catch(java.io.IOException fnfe)
		{
			MatrixUtil.toLog("SRBConnection-IOException-END"+fnfe, MatrixUtil.ERROR_LOG);
		}
		catch(Exception e)
		{
			MatrixUtil.toLog("SRBConnection-Exception-END"+e, MatrixUtil.ERROR_LOG);
		}
		
	}
	
	public static void main(String args[])
	{
		try{
			System.out.println("SRBCOnnection  :MAIN-BEGIN");		
		SRBConnection srbConn = new SRBConnection();
		UserInfo srbUser = JaxbUtil.createUserInfo(args[2],args[3],args[4],args[6],args[5]);
		VirtualOrganizationInfo virtualOrg = JaxbUtil.createVirtualOrganizationInfo(args[0], Integer.parseInt(args[1]));
		
		srbConn.createSRBConnection(srbUser,virtualOrg);
		System.out.println("SRBCOnnection  :MAIN-END");
		}
		catch(Exception e)
		{
			System.out.println("Exception :"+e);
		}
	}
	
}	