/* ====================================================================
 * SDSC Matrix Software License, Version 1.0
 *
 * Copyright (c) 2003 San Diego Supercomputer Center (SDSC), San Diego, USA.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that ALL the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        SDSC Matrix Project (http://www.npaci.edu/DICE/SRB/matrix/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SDSC" and "SDSC Matrix" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact arun@sdsc.edu or srb-chat@sdsc.edu.
 *
 * 5. Products derived from this software may not be called "SDSC",
 *    nor may "SDSC" appear in their name, without prior written
 *    permission of the SDSC.
 *
 * 6. License is not granted for commercial resale, in whole or in part, 
 *	  without the prior written permission from SDSC.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SDSC OR
 * ITS CONTRIBUTORS (SDSC Matrix Team) BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software is intended to consist of voluntary contributions made by many
 * individuals for the SDSC Matrix Project.
 *
 * SDSC Matrix.  
 * Designer & Architect: Arun swaran Jagatheesan (arun@sdsc.edu)
 *
 * Authors (Credits 02/SEP/2003):
 * Allen Ding (alding@sdsc.edu)
 * Arun Jagatheesan (arun@sdsc.edu)
 * Reena Mathew	(rmathew@sdsc.edu)
 * Erik Vandekieft (evk@sdsc.edu)
 */



package edu.sdsc.matrix.srb.core;

import java.io.FileInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.soap.*;
import javax.activation.*;
import edu.sdsc.matrix.srb.util.*;
import edu.sdsc.matrix.srb.parser.*;
import edu.sdsc.matrix.srb.operations.MatrixTransactionHandler;

/**
 * Class that handles all requests for the Matrix Server. 
 * Class redirects the request to the appropriate handler. 
 *  
 * CAUTION: this code was written using the beta version of JAXB 
 * which has bugs: You must be sure to put your jaxb.properties file
 * in the directory that contains the jaxb .class files (the one
 * above the "/impl" directory, and you must not package these jaxb 
 * .class files into a jarfile if you want the server to run on tomcat.
 * 
 *  @author evk,rmathew
 *
 * 
 */

public class SRBProcessor 
{

	/**
	 * Unmarshaller for unmarshalling JAXB Objects
	 */
    private Unmarshaller unmarshaller;
    
    /**
     * Marshaller for marshalling JAXB Objects
     */
    private Marshaller marshaller;
    
    /**
     * Store represents the repository of items for the Matrix Server
     */
    private SRBStore store;
    
    /**
     * JAXB Object to create JAXB Objects
     */
    private ObjectFactory objFactory;
    
    /**
     * Used for creating transaction ids
     */
    private long transactionIDCounter;

	/**
	 * Public Constructor
	 */
    public SRBProcessor() throws JAXBException 
    {
		JAXBContext jc = JAXBContext.
		newInstance("edu.sdsc.matrix.srb.parser");
		unmarshaller = jc.createUnmarshaller();
		marshaller = jc.createMarshaller();
		
		store = new SRBStore();
		objFactory = new ObjectFactory();
		transactionIDCounter = 0;
		transactionIDCounter = System.currentTimeMillis();  
    }

	/**
	 * Processes the Incoming Request
	 * @param requestxml Incoming Request in XML format
	 * @param requestAttachments List of Attachments for a request
	 * @param responseAttachments List of Attachement for the response
	 * @return Source for response to request
	 */
    public Source process(Source requestxml, Iterator requestAttachments, 
    					  List responseAttachments) throws JAXBException 
    {
		//begin ARUN-Change-Aug21
		
		MatrixUtil.toLog("Got a request at Matrix Processor",MatrixUtil.DEBUG_LOG);
		//end ARUN-Change-Aug21
		// we want the "unmarshal" call outside of the try block so that we can differentiate
		// between JAXBExceptions that are the server's fault versus the client's fault.
		// if it's the server's fault, print a stack trace and return null; the calling logic
		// will form a SOAP fault saying that the server suffered an internal error.
		// if it's the client's request that is malformed, we'll detect that in the calling logic 
		// and form a SOAP Fault that tells the client what the JAXB exception was.
		
		DataGridRequest dgrequest = (DataGridRequest) unmarshaller.unmarshal(requestxml);
        try 
        {
	    	DataGridResponse dgresponse = objFactory.createDataGridResponse();

	    	if(dgrequest.getTransaction() != null) 
	    	{ // it's a new flow to execute
	       		TransactionAcknowledgment transAck = handleTransaction(dgrequest, requestAttachments);
				dgresponse.setTransactionAcknowledgement(transAck);
	    	}
	    	else 
	    	{ // it's a status request
				TransactionStatusResponse statusResponse = 
				handleTransactionStatus(dgrequest.getTransactionStatusQuery(), responseAttachments);
				dgresponse.setTransactionStatusResponse(statusResponse);
	    	}
	    	dgresponse.setDocMeta(JaxbUtil.createDocMeta());
	    	return convertToSource(dgresponse); // have to create a source object

        } 
        catch(JAXBException e)
        {
			//begin ARUN-Change-Aug21
			//log this always = 100?
			MatrixUtil.toLog("\n\n\nError Processing a request: strange exception:\n"+e.toString(),MatrixUtil.FATAL_LOG);
			//end ARUN-Change-Aug21
            e.printStackTrace(); 
	    	return null;
		}
    }
    
	//	Added by Reena Mathew on 18 August 2003
	/**
	 * This method will convert the attachment iterator to a hashtable
	 *	and maintain it in SRBStore
	 * The operations can retrieve the attachment from the SRBStore
	 * 
	 * @param transactionID Transaction ID of request
	 * @param attachmentItr List of request Attachments 
	 */ 
    private void handleTransactionInputAttachments(String transactionID, Iterator attachmentItr)
    {
    	String contentID= null;
		AttachmentPart currentAttachment = null;
		MatrixUtil.toLog(transactionID,"SRBProcessor - handleTransactionInputAttachments()-BEGIN",MatrixUtil.DEBUG_LOG);
		MatrixUtil.toLog(transactionID,"SRBProcessor - handleTransactionInputAttachments()-transactionID:"+transactionID,MatrixUtil.DEBUG_LOG);
		if(attachmentItr!=null)
		{
			MatrixUtil.toLog(transactionID,"SRBProcessor - handleTransactionInputAttachments()",MatrixUtil.DEBUG_LOG);	
			while(attachmentItr.hasNext())
			{
				currentAttachment = (AttachmentPart) attachmentItr.next();
				contentID = currentAttachment.getContentId();
				if(currentAttachment!=null && contentID!=null)
				{
					MatrixUtil.toLog(transactionID,"SRBProcessor - handleTransactionInputAttachments()-transactionID.contentID:"+transactionID+"."+contentID + " attachment stored",MatrixUtil.DEBUG_LOG);
					store.addInputAttachment(transactionID,contentID,currentAttachment);			
				}
			}
		}
		MatrixUtil.toLog(transactionID,"SRBProcessor - handleTransactionInputAttachments()-END",MatrixUtil.DEBUG_LOG);		
    	
    }

	/**
	 * Should return a transaction ID.
	 * Create a DataGridResponse, populate it with that transaction ID.
	 * Marshall it and return it in XML form for JAXM to send on wire.
	 * @param dgrequest Incoming Request
	 * @param requestAttachments Attachments for Request
	 * @return Transaction Acknowledgement which contains the request's transaction id
	 */    
    private TransactionAcknowledgment handleTransaction(DataGridRequest dgrequest, 
														Iterator requestAttachments) 
	throws JAXBException 
	{
		String transactionID = getNextTransactionID();
		MatrixTransactionHandler handler = new MatrixTransactionHandler(dgrequest, transactionID, store);
		handler.setAttachmentIterator(requestAttachments);
	
		//	Added by Reena Mathew on 18 August 2003
		handleTransactionInputAttachments(transactionID,requestAttachments);
		handler.start();
	

		TransactionAcknowledgment transAck = objFactory.createTransactionAcknowledgment();
		transAck.setRequestState(0);
		transAck.setTransactionID(transactionID);

		return transAck;
    }
    
    
    /**
     * Handles the status check for a flow in a transaction
     * @param transactionID Transaction Id of flow
     * @param flowIdForStatus Flow Id for check
     * @param flowQuery Flow Query Request
     * @return Flow Status Response
     */
    private FlowStatusResponse handleStatusForFlow(String transactionID , 
    											   String flowIdForStatus , 
    											   FlowStatusQuery flowQuery) 
    												throws JAXBException
    {
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatusForFlow():BEGIN", MatrixUtil.DEBUG_LOG);
		FlowStatusResponse flowresponse = null;
		FlowStatusResponse flowresponseFromStore = null;
		String flowId = null;
		boolean flowStatusFoundInStore = false; 
		List requestSteps = null;
		List responseSteps = null;
		
		flowId = flowIdForStatus ; //flowQuery.getFlowID();
		
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatusForFlow():TXNID.FLOWID:"+transactionID+"."+flowId, MatrixUtil.DEBUG_LOG);
		
		// Check for FLOW STOP
		//if(CHECK FOR FLOW STOP)
			//handleFlowStop(transactionID,flowId,flowQuery);
		
		
		flowresponse = objFactory.createFlowStatusResponse();
		flowresponseFromStore = (FlowStatusResponse)store.getMatrixFlowStatus(transactionID,flowId);
		
		//	Update New Response With that From Store - Duplicate Entries
		 if(flowresponseFromStore!=null)
		 {
			 flowresponse.setFlowID(flowresponseFromStore.getFlowID());
			 flowresponse.setStartTime(flowresponseFromStore.getStartTime());
			 flowresponse.setEndTime(flowresponseFromStore.getEndTime());
			 flowresponse.setAbortRequested(flowresponseFromStore.isAbortRequested());			
			 flowresponse.setStatusCode(flowresponseFromStore.getStatusCode());
			 MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatusForFlow():txnid.flowId-:"+transactionID+"."+flowId+"FLOW RESPONSE FOUND - CODE:"+flowresponseFromStore.getStatusCode(),MatrixUtil.DEBUG_LOG);
			flowStatusFoundInStore = true;
		 }
		else
		{
			flowresponse.setFlowID(flowId);
			flowresponse.setStatusCode(MatrixCodes.FLOW_NOT_FOUND_OR_NOT_STARTED);
			flowStatusFoundInStore = false;
			MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatusForFlow():txnid.flowId-:"+transactionID+"."+flowId+"FLOW RESPONSE NOT FOUND-CODE:"+flowresponse.getStatusCode(), MatrixUtil.DEBUG_LOG);				
		}
		
		if(flowStatusFoundInStore)
		{
		   responseSteps = flowresponse.getStepStatusResponse();			
		   if(flowQuery!=null)
		   {	   
		   	   requestSteps = flowQuery.getStepStatusQuery();
		   	   if(requestSteps.isEmpty())
		   	   { // look up status of EVERY step for this transID
					appendStatusOfEveryStep(transactionID, flowId,responseSteps);
			   //if(!appendStatusOfEveryStep(transactionID, flowId,responseSteps)) 
			   //statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
		       }
		      else
		      { 
		      	// or only through the specified ones
				appendStatusOfCertainSteps(transactionID, flowId,requestSteps, responseSteps);
			   //if(!appendStatusOfCertainSteps(transactionID, flowId,requestSteps, responseSteps)) 
			   //statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
		   	  }
		   }
		   else
		   {
				appendStatusOfEveryStep(transactionID, flowId,responseSteps);
		   }
		   

		}	 
		
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatusForFlow():END",  MatrixUtil.DEBUG_LOG);
		return(flowresponse);
    }
    
    /**
     * Handles the process of stopping a transaction
     * This method will just set a flag in the SRB Store which
     * will be checked before the transaction starts. 
     * @param transactionID Transaction to be stopped
     * @param txnQuery Transaction Query Request 
     */
    private void handleTransactionStop(String transactionID,TransactionStatusQuery txnQuery)
	throws JAXBException
    {
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleTransactionStop():BEGIN", MatrixUtil.DEBUG_LOG);
    	TransactionStatusResponse txnStatus =  (TransactionStatusResponse)
												store.getTransactionStatus(transactionID);
	
		if(txnStatus==null) // Create Entry in Store to stop transaction
		{
			txnStatus = objFactory.createTransactionStatusResponse();														
			txnStatus.setTransactionID(transactionID);			
		}
		txnStatus.setAbortRequested(true);
    	
		store.setTransactionStatus(transactionID,txnStatus);
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleTransactionStop():END",MatrixUtil.DEBUG_LOG);
    }
    
    /**
     * Handles the process of stopping a flow. 
     * This method will just set a flag in the SRB Store which
     * will be checked before the flow starts. 
     * @param txnID Transaction ID of flow
     * @param flowID Flow to be stopped
     * @param flowQuery Flow Query Request
     */
    private void handleFlowStop(String txnID,String flowID,FlowStatusQuery flowQuery)
	throws JAXBException
    {
		MatrixUtil.toLog(txnID,"SRBProcessor-handleFlowStop():BEGIN", MatrixUtil.DEBUG_LOG);
		FlowStatusResponse flowStatus =  (FlowStatusResponse)
									store.getMatrixFlowStatus(txnID,flowID);
		if(flowStatus==null) // Create Entry in Store to stop transaction
		{
			flowStatus = objFactory.createFlowStatusResponse();													
			flowStatus.setFlowID(flowID);			
		}
		flowStatus.setAbortRequested(true);

		store.setMatrixFlowStatus(txnID,flowID,flowStatus);
		MatrixUtil.toLog(txnID,"SRBProcessor-handleFlowStop():END", MatrixUtil.DEBUG_LOG);
    }
    
    
	/**
	 * Handles the process of stopping a step. 
	 * This method will just set a flag in the SRB Store which
	 * will be checked before the step starts. 
	 * @param txnID Transaction ID of flow
	 * @param flowID Flow ID of step
	 * @param stepID Step ID to be stopped 
	 * @param stepQuery Step Status Query Request
	 */
    private void handleStepStop(String txnID,String flowID, String stepID,StepStatusQuery stepQuery)
	throws JAXBException
    {
		MatrixUtil.toLog(txnID,"SRBProcessor-handleStepStop():BEGIN", MatrixUtil.DEBUG_LOG);
		StepStatusResponse stepStatus =  (StepStatusResponse)store.getStepStatus(txnID,flowID,stepID);
		if(stepStatus!=null)
		{
			stepStatus = objFactory.createStepStatusResponse();		
			stepStatus.setStepID(stepID);
		}
		stepStatus.setAbortRequested(true);
		store.setStepStatus(txnID, flowID,stepID,stepStatus);
		MatrixUtil.toLog(txnID,"SRBProcessor-handleStepStop():END", MatrixUtil.DEBUG_LOG);
    }
    
	/**
		 * Changes by Reena Mathew on 20 August 2003
		 * This method is not being used anymore
		 * This status query for a transaction has been split into the following
		 * 	- handleTransactionStatus
		 * 	- handleStatusForFlow
		 *  - appendStatusOfEveryStep
		 * 	- appendStatusOfCertainSteps
		 * 	- handleTransactionStop
		 *  - handleFlowStop
		 *  - handleStepStop
		 */
    
    private TransactionStatusResponse handleTransactionStatus(TransactionStatusQuery txnQuery,
    													List responseAttachments)
														throws JAXBException
    {
		MatrixUtil.toLog("SRBProcessor-handleTransactionStatus():BEGIN", MatrixUtil.DEBUG_LOG);	
		TransactionStatusResponse statusResponse = null; 
		TransactionStatusResponse statusResponseFromStore = null;
		String transactionID = null; 
		boolean transactionFound = false;
		List flowQueryList = null;
		List stepQueryList = null; 
		FlowStatusQuery flowQuery = null;
		FlowStatusResponse flowStatusResponse = null;
		List responseAttachmentsInStore = null;
		
		// Create Status Response For Return Value 
		statusResponse = objFactory.createTransactionStatusResponse();
		
		// Extract TXN ID 
		transactionID = txnQuery.getTransactionID();
		
		// Check for Transaction STOP
		//if(CHECK FOR STOP)
			//handleTransactionStop(transactionID,txnQuery);
		
		// Retrieve TXN's status from STORE
		statusResponseFromStore = (TransactionStatusResponse)store.getTransactionStatus(transactionID);
		
		
		// Update New Response With that From Store - Duplicate Entries
		 if(statusResponseFromStore!=null)
		 {
		 	 statusResponse.setTransactionID(statusResponseFromStore.getTransactionID());
			 statusResponse.setStartTime(statusResponseFromStore.getStartTime());
			 statusResponse.setEndTime(statusResponseFromStore.getEndTime());
			 statusResponse.setAbortRequested(statusResponseFromStore.isAbortRequested());			
			 statusResponse.setStatusCode(statusResponseFromStore.getStatusCode());
			 transactionFound = true;
		 }
		 else
		 {
			 statusResponse.setTransactionID(transactionID);
			 statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
			 transactionFound = false;
		 }
		 
		 if(transactionFound)
		 {
			flowQueryList = txnQuery.getFlowStatusQuery();
			if(flowQueryList!=null && flowQueryList.size()>0)
			{
				// TXN Query has flow status queries
				for(int flowCount=0;flowCount<flowQueryList.size();flowCount++)
				{
					flowQuery = (FlowStatusQuery)flowQueryList.get(flowCount);
					flowStatusResponse = handleStatusForFlow(transactionID,flowQuery.getFlowID(),flowQuery);
					statusResponse.getFlowStatusResponse().add(flowStatusResponse);
				}
			}
			else
			{
				// TXN Query has no flow status queries
				// Retrieve All Flow Status Queries for TXN or not ??????
				String[] listOfFlowIds = store.getTransactionFlowIDs(transactionID);
				if(listOfFlowIds!=null)
				{
					for(int index=0;index<listOfFlowIds.length;index++)
					{
							String flowId= listOfFlowIds[index];
							flowStatusResponse = handleStatusForFlow(transactionID,flowId,null);
							statusResponse.getFlowStatusResponse().add(flowStatusResponse);
					}
				}			
				
				
			}
			
			// Add Attachment For Transaction
			responseAttachmentsInStore = store.getAttachmentList(transactionID);
			if(responseAttachmentsInStore != null)
			{
					responseAttachments.addAll(responseAttachmentsInStore);
			}

		 }
		
		
		MatrixUtil.toLog(transactionID,"SRBProcessor-handleTransactionStatus():END", MatrixUtil.DEBUG_LOG);
		return statusResponse;	
    }


	/**
	 * Changes by Reena Mathew on 20 August 2003
	 * This method is not being used anymore
	 * This method has been replaced with
	 * 	- handleTransactionStatus	
	 */
    private TransactionStatusResponse handleStatus(TransactionStatusQuery query, 
						   List responseAttachments) 
	throws JAXBException {
	MatrixUtil.toLog("SRBProcessor-handleStatus:BEGIN", MatrixUtil.DEBUG_LOG);
	
			
	String transactionID = query.getTransactionID();

	// create status response
	TransactionStatusResponse statusResponse = objFactory.createTransactionStatusResponse();
	TransactionStatusResponse statusResponseFromStore = (TransactionStatusResponse)store.getTransactionStatus(transactionID);
	
	// Update New Response With that From Store - Duplicate Entries
	if(statusResponseFromStore!=null)
	{
		statusResponse.setTransactionID(statusResponseFromStore.getTransactionID());
		statusResponse.setStartTime(statusResponseFromStore.getStartTime());
		statusResponse.setEndTime(statusResponseFromStore.getEndTime());
		statusResponse.setAbortRequested(statusResponseFromStore.isAbortRequested());			
		statusResponse.setStatusCode(statusResponseFromStore.getStatusCode());
	}
	else
	{
		statusResponse.setTransactionID(transactionID);
		statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
	}
	
	//FlowStatusResponse flowresponse = objFactory.createFlowStatusResponse();
	//flowresponse.setFlowID("0"); // SHOULD FIX THIS WHEN WE ACTUALLY HANDLE MULTIPLE FLOWS
	MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:transactionID:"+transactionID, MatrixUtil.DEBUG_LOG);	
	String[] listOfFlowIds = store.getTransactionFlowIDs(transactionID);
	if(listOfFlowIds!=null)
	{
		for(int index=0;index<listOfFlowIds.length;index++)
		{
			String flowId= listOfFlowIds[index];//"0";
			MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:txnid.flowId:"+transactionID+"."+flowId, MatrixUtil.DEBUG_LOG);
			FlowStatusResponse flowresponse = objFactory.createFlowStatusResponse();
			FlowStatusResponse flowresponseFromStore = null; 
			flowresponseFromStore = (FlowStatusResponse)store.getMatrixFlowStatus(transactionID,flowId);
						
			// Update New Response With that From Store - Duplicate Entries
			if(flowresponseFromStore!=null)
			{
				flowresponse.setFlowID(flowresponseFromStore.getFlowID());
				flowresponse.setStartTime(flowresponseFromStore.getStartTime());
				flowresponse.setEndTime(flowresponseFromStore.getEndTime());
				flowresponse.setAbortRequested(flowresponseFromStore.isAbortRequested());			
				flowresponse.setStatusCode(flowresponseFromStore.getStatusCode());
				MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:txnid.flowId-:"+transactionID+"."+flowId+"FLOW RESPONSE FOUND - CODE:"+flowresponseFromStore.getStatusCode(), MatrixUtil.DEBUG_LOG);
				
				// Changed by Reena Mathew on 18 August 2003 
				// Fixed bug of status showing FLOW NOT FOUND and STEP STARTED
				// STATUS OF STEPS ONLY IF FLOW FOUND
				List requestSteps = ((FlowStatusQuery)query.getFlowStatusQuery().get(index)).getStepStatusQuery();
				List responseSteps = flowresponse.getStepStatusResponse();
	
				if(requestSteps.isEmpty()) { // look up status of EVERY step for this transID
					if(!appendStatusOfEveryStep(transactionID, flowId,responseSteps)) 
					statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
				}
				else { // or only through the specified ones
					if(!appendStatusOfCertainSteps(transactionID, flowId,requestSteps, responseSteps)) 
					statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
				}
				
				
			}
			else
			{
				flowresponse.setFlowID(flowId);
				flowresponse.setStatusCode(MatrixCodes.FLOW_NOT_FOUND_OR_NOT_STARTED);
				MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:txnid.flowId-:"+transactionID+"."+flowId+"FLOW RESPONSE NOT FOUND-CODE:"+flowresponse.getStatusCode(), MatrixUtil.DEBUG_LOG);				
			}
						
			statusResponse.setTransactionID(transactionID);
			
			/*	
			List requestSteps = ((FlowStatusQuery)query.getFlowStatusQuery().get(index)).getStepStatusQuery();
			List responseSteps = flowresponse.getStepStatusResponse();
			
			if(requestSteps.isEmpty()) { // look up status of EVERY step for this transID
			    if(!appendStatusOfEveryStep(transactionID, flowId,responseSteps)) 
				statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
			}
			else { // or only through the specified ones
			    if(!appendStatusOfCertainSteps(transactionID, flowId,requestSteps, responseSteps)) 
				statusResponse.setStatusCode(MatrixCodes.TRANSACTION_NOT_FOUND_OR_NOT_STARTED);
			}
			*/
			MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:ADD FLOW STATUS RESPONSE : flowId:"+flowId, MatrixUtil.DEBUG_LOG);
			statusResponse.getFlowStatusResponse().add(flowresponse);	
		}
	}
	
	List responseAttachmentsInStore = store.getAttachmentList(transactionID);
	if(responseAttachmentsInStore != null){
	    responseAttachments.addAll(responseAttachmentsInStore);
	}
	
	
	MatrixUtil.toLog(transactionID,"SRBProcessor-handleStatus:END", MatrixUtil.DEBUG_LOG);	
	return statusResponse;
    }

	/**
	 * Retrieves the status of every step for a flow
	 * @param transactionID Transaction ID of flow
	 * @param flowId Flow ID for which step statuses are requested
	 * @param responseSteps Step Status Responses
	 * @return Flag to indicate if any steps statuses were returned
	 */
    private boolean appendStatusOfEveryStep(String transactionID, String flowId,List responseSteps) 
	throws JAXBException 
	{
		// first check if this transactionID exists
		MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfEveryStep:BEGIN", MatrixUtil.DEBUG_LOG);	
		MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfEveryStep:flowId:"+flowId, MatrixUtil.DEBUG_LOG);
		String[] stepids = store.getTransactionStepIDs(transactionID,flowId);
		if(stepids == null) 
		{
	    	return false;
		}

		for(int i=0; i<stepids.length; i++)
		{
		
			MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfEveryStep():TXNID.FLOWID.STEPID:"+transactionID+"."+flowId+"."+stepids[i], MatrixUtil.DEBUG_LOG);
		
			MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfEveryStep:appending stepname: " + stepids[i],MatrixUtil.DEBUG_LOG);
	    	StepStatusResponse stepStatusResponse = store.getStepStatus(transactionID,flowId, stepids[i]);
	    
	    	if(stepStatusResponse == null) 
	    	{
				MatrixUtil.toLog(transactionID,"creating error step status response",MatrixUtil.DEBUG_LOG);
				responseSteps.add(JaxbUtil.createErrorStepStatusResponse(stepids[i]));
	    	}
	    	else 
	    	{
				MatrixUtil.toLog(transactionID,"appending regular step status response",MatrixUtil.DEBUG_LOG);
				MatrixUtil.toLog(transactionID,"step status response tostring(): " + stepStatusResponse.toString(),MatrixUtil.DEBUG_LOG);
				responseSteps.add(stepStatusResponse);
	    	}
		}
		MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfEveryStep:END", MatrixUtil.DEBUG_LOG);	
		return true;
    }

	/**
	 * Retrieves the status of specified step for a flow
	 * @param transactionID Transaction ID of flow
	 * @param flowId Flow ID for which step statuses are requested
	 * @param requestSteps Steps for which status requested
	 * @param responseSteps Step Status Responses
	 * @return Flag to indicate if any steps statuses were returned
	 */
    private boolean appendStatusOfCertainSteps(String transactionID, String flowId,List requestSteps, List responseSteps) 
	throws JAXBException 
	{
		// first check if this transactionID exists
		MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfCertainSteps:BEGIN", MatrixUtil.DEBUG_LOG);	
		String[] stepids = store.getTransactionStepIDs(transactionID,flowId);
		if(stepids == null) 
		{
	    	return false;
		}

		Iterator it = requestSteps.iterator();
		while(it.hasNext())
		{
	    	StepStatusQuery ss = (StepStatusQuery)it.next();
		
			MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfCertainSteps():TXNID.FLOWID.STEPID:"+transactionID+"."+flowId+"."+ss.getStepID(), MatrixUtil.DEBUG_LOG);	
	    	if(ss.getQueryType() != null) 
	    	{
				StepStatusResponse stepStatusResponse = store.getStepStatus(transactionID,flowId, ss.getStepID());
				if(stepStatusResponse == null) 
				{
		    		responseSteps.add(JaxbUtil.createErrorStepStatusResponse(ss.getStepID()));
				}
				else 
				{
		    		responseSteps.add(stepStatusResponse);
				}
	    	}
	    	else if(ss.getStop() != null) 
	    	{ 
				// NOT SUPPORTED: stopping the process
				//handleStepStop(transactionID,flowId, ss.getStepID(),ss);
				StepStatusResponse stepStatusResponse = store.getStepStatus(transactionID,flowId, ss.getStepID());
				responseSteps.add(stepStatusResponse);
	    	}
		}
		MatrixUtil.toLog(transactionID,"SRBProcessor-appendStatusOfCertainSteps:END", MatrixUtil.DEBUG_LOG);
		return true;
    }

    /** 
     * Marshall the datagridresponse, and turn it into a "Source" object
     * @param response Response to be returned
     * @return Source format of Response
     * 
     */
    private Source convertToSource(DataGridResponse response) 
	throws JAXBException 
	{
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		marshaller.marshal(response, os);
		ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
		return new StreamSource(is);
    }

	/**
	 * Return the new Transaction ID
	 * @return New Transaction id to be used
	 */
    private String getNextTransactionID() 
    {
		return "" + transactionIDCounter++;
    }
    
	
	
	//begin ARUN-Change-Aug21
	/* for internal testing 
    public static void main( String[] args ) {

	String ATTACH_FILENAME = "test.txt";
	String ATTACH_MIMENAME= "attachname";
	String XML_SAMPLE_NAME = "sampleDataGridRequest.xml";
	try {
		System.out.println("IN MAIN");
	    SRBProcessor p = new SRBProcessor();
            MessageFactory mf = MessageFactory.newInstance();
            SOAPMessage msg = mf.createMessage();
	    FileDataSource ds = new FileDataSource(ATTACH_FILENAME);
		FileDataSource ds2 = new FileDataSource("test.xml");
	    DataHandler dh = new DataHandler(ds);
		DataHandler dh1 = new DataHandler(ds2);
	    AttachmentPart attachment = msg.createAttachmentPart(dh);
	    attachment.setContentId("attachname1");
		AttachmentPart attachment2 = msg.createAttachmentPart(dh1);
		attachment2.setContentId("attachname2");
	    ArrayList attachmentList = new ArrayList();
	    attachmentList.add(attachment);
		attachmentList.add(attachment2);

		List tempResponseList =null;
		p.process(new StreamSource(new FileInputStream(XML_SAMPLE_NAME)), attachmentList.iterator(),tempResponseList);
        } catch( Exception e ) {
            e.printStackTrace(); // means i couldn't even instantiate a new SRBProcessor
	} 
	}//main
	*/
	//end ARUN-Change-Aug21 
}
