/* ====================================================================
 * SDSC Matrix Software License, Version 1.0
 *
 * Copyright (c) 2003 San Diego Supercomputer Center (SDSC), San Diego, USA.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that ALL the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        SDSC Matrix Project (http://www.npaci.edu/DICE/SRB/matrix/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SDSC" and "SDSC Matrix" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact arun@sdsc.edu or srb-chat@sdsc.edu.
 *
 * 5. Products derived from this software may not be called "SDSC",
 *    nor may "SDSC" appear in their name, without prior written
 *    permission of the SDSC.
 *
 * 6. License is not granted for commercial resale, in whole or in part, 
 *	  without the prior written permission from SDSC.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SDSC OR
 * ITS CONTRIBUTORS (SDSC Matrix Team) BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software is intended to consist of voluntary contributions made by many
 * individuals for the SDSC Matrix Project.
 *
 * SDSC Matrix.  
 * Designer & Architect: Arun swaran Jagatheesan (arun@sdsc.edu)
 *
 * Authors (Credits 02/SEP/2003):
 * Allen Ding (alding@sdsc.edu)
 * Arun Jagatheesan (arun@sdsc.edu)
 * Reena Mathew	(rmathew@sdsc.edu)
 * Erik Vandekieft (evk@sdsc.edu)
 */



package edu.sdsc.matrix.srb.core;

import java.util.Hashtable;
import java.util.List;
import edu.sdsc.matrix.srb.parser.TransactionStatusResponse;
import edu.sdsc.matrix.srb.parser.FlowStatusResponse;
import edu.sdsc.matrix.srb.parser.StepStatusResponse;
import edu.sdsc.matrix.srb.util.MatrixUtil;

// Added by Reena on 14 May 2003 - Attachments
import java.util.ArrayList;

/**
 * 
 * Class stores items required for Matrix Server 
 * These items are stored in hashtables. 
 * These hashtables will later on be replaced by a persistence store
 *  
 * @author evk,rmathew
 *
 * 
 */
public final class SRBStore 
{

	/**
	 *  Key=TransactionID.FlowID.StepID , Value=StepStatusResponse(Jaxb)
	 */
    private Hashtable steps;
    
    /**
     * Key =TransactionID.FlowID ,Value - List of Step IDs (String[])
     */
    private Hashtable transactions; 
    
	/**
	 * For storing tickets Key=TicketID,Value=GridTicketInfo(Jaxb)
	 */
    private Hashtable tickets; 
    
    
	/**
	 *  Added by Reena on 14 May 2003 to maintain Attachments for a Transaction
	 *  Key=TransactionID , Value= AttachmentPart (From SOAP)
	 */
    private Hashtable attachments; 
    
	
	/**
	 * Added by Reena on 19 June 2003 to maintain flows for a Transaction
	 *  Key=TransactionID.FlowID ,Value =FlowStatusResponse(Jaxb)
	 */
    private Hashtable matrixFlow; 
    
    
	/**
	 *  Key =TransactionID, Value = TransactionStatusResponse(Jaxb)
	 */
	private Hashtable transactionStatus; 


	/**
	 * Key =TransactionID, Value = List of Flow IDS (String[])
	 */
	private Hashtable transactionFlows; 
	
	
	
	/**
	 *  Added by Reena Mathew on 18 August 2003 
	 *  Hashtable maintains the input attachments for a transaction
	 *  Key = TransactionId.ContentID , Value= Attachment
	 */
	private Hashtable transactionInputAttachments; 


	/**
	 * Public Constructor
	 * Initializes all the hashtables for the store
	 */
    public SRBStore() 
    {
		steps = new Hashtable();
		transactions = new Hashtable();
		tickets = new Hashtable();
		
		// Added by Reena on 14 May 2003
		attachments = new Hashtable(); 
	
		//	Added by Reena on 19 June 2003
		// NOTE - Transaction - > N Flows -> N Steps
		transactionStatus = new Hashtable();
		matrixFlow = new Hashtable();
	
		transactionFlows = new Hashtable();
	
		//	Added by Reena Mathew on 18 August 2003 
		transactionInputAttachments = new Hashtable();
    }
    
    /**
     * Sets the list of flow ids for a transaction
     * @param transactionID Transaction ID
     * @param user User of transaction
     * @param flowIDs List of flow ids for the transaction
     */
	public void setTransactionFlowIDs(String transactionID, String user, String[] flowIDs)
	{
		MatrixUtil.toLog(transactionID,"SRBStore - setTransactionFlowIDs()-transactionID :"+transactionID,MatrixUtil.DEBUG_LOG);
		transactionFlows.put(transactionID, flowIDs);
	}

	 
	/**
	 * Retrieves the list of flow ids for a transaction
	 * @param transactionID Transaction ID
	 * @return List of Flows for the transaction
	 */
	public String[] getTransactionFlowIDs(String transactionID)
	{
		MatrixUtil.toLog(transactionID,"SRBStore - getTransactionFlowIDs() -transactionID:"+transactionID,MatrixUtil.DEBUG_LOG);
	   return (String[])transactionFlows.get(transactionID);
	}
    
    
    /**
     * Sets the list of step ids for a particular flow in a transaction
     * @param transactionID Transaction ID
     * @param flowID Flow ID
     * @param user User
     * @param stepIDs List of Step ids for the flow
     */
    public void setTransactionStepIDs(String transactionID,String flowID, String user, String[] stepIDs)
    {
		MatrixUtil.toLog(transactionID,"SRBStore - setTransactionStepIDs() -transactionID-flowid:"+transactionID+"."+flowID,MatrixUtil.DEBUG_LOG);
		transactions.put(createkey(transactionID,flowID), stepIDs);
    }
	
	/**
	 * Set the Status for a Step
	 * @param transactionID Transaction ID
	 * @param flowId Flow ID
	 * @param stepID Step ID
	 * @param status Status for Step 
	 */
    public void setStepStatus(String transactionID, String flowId,String stepID, StepStatusResponse status) 
    {
		MatrixUtil.toLog(transactionID,"SRBStore - setStepStatus() -transactionID-flowid:"+transactionID+"."+flowId,MatrixUtil.DEBUG_LOG);
		if(status!=null)
		{
				MatrixUtil.toLog(transactionID,"SRBStore - getStepStatus() -transactionID.flowid-STEP RESPONSE:"+status.getStatusCode(),MatrixUtil.DEBUG_LOG);
		}
		steps.put(createkey(transactionID+"."+flowId, stepID), status);
    }

	/**
	 * Retrieve the Step ids for a flow in a transaction
	 * @param transactionID Transaction ID
	 * @param flowID Flow ID
	 * @return List of Step IDS for a flow in a transaction
	 */
    public String[] getTransactionStepIDs(String transactionID,String flowID)
    {
		MatrixUtil.toLog(transactionID,"SRBStore - getTransactionStepIDs() -transactionID.flowId:"+transactionID+"."+flowID,MatrixUtil.DEBUG_LOG);
		return (String[])transactions.get(createkey(transactionID,flowID));
    }

	/**
	 * Retrieve the status for a step 
	 * @param transactionID Transaction ID
	 * @param flowId Flow ID
	 * @param stepID Step ID
	 * @return Status For Step
	 */
    public StepStatusResponse getStepStatus(String transactionID, String flowId,String stepID)
    {
		MatrixUtil.toLog(transactionID,"SRBStore - getStepStatus() -transactionID.flowid:"+transactionID+"."+flowId,MatrixUtil.DEBUG_LOG);
		StepStatusResponse response = (StepStatusResponse)steps.get(createkey(transactionID+"."+flowId, stepID));
		if(response!=null)
		{
			MatrixUtil.toLog(transactionID,"SRBStore - getStepStatus() -transactionID.flowid-STEP RESPONSE:"+response.getStatusCode(),MatrixUtil.DEBUG_LOG);
		}
		return response;
    }

	/**
	 * Create Key for using in a hashtable
	 * This appends the keyPart1 with keyPart2
	 * @param transactionID Transaction ID
	 * @param stepID Step ID
	 * @return Key created using the values passed in
	 */
    private String createkey(String keyPart1, String keyPart2)
    {
		MatrixUtil.toLog(keyPart1,"SRBStore - createkey() -keyPart1-keyPart2:"+keyPart1+"-"+keyPart2,MatrixUtil.DEBUG_LOG);
		return keyPart1 + "." + keyPart2;
    }
    
    /**
     * Save ticket in Ticket Store
     * @param ticketId Ticket ID
     * @param ticket Ticket for storing in ticket store
     * 
     */
    public void saveTicket(String ticketId,Object ticket)
    {
		MatrixUtil.toLog("SRBStore - saveTicket() -ticketId:"+ticketId,MatrixUtil.DEBUG_LOG);
    	tickets.put(ticketId,ticket);
    }
    
    /**
     * Retrieve Ticket given a ticket id
     * @param ticketId Ticket ID
     * @return Ticket from store
     */
    public Object retrieveTicket(String ticketId)
    {
		MatrixUtil.toLog("SRBStore - retrieveTicket() -ticketId:"+ticketId,MatrixUtil.DEBUG_LOG);
    	return(tickets.get(ticketId));
    }
    
	// Added by Reena on 14 May 2003
	/**
	 * Save Response Attachment in Response Attachment Store
	 * @param transactionId Transaction ID
	 * @param attachment Attachment to save in Attachment Store
	 */
	public void saveAttachment(String transactionId,Object attachment)
	{
		MatrixUtil.toLog(transactionId,"SRBStore - saveAttachment() -transactionId:"+transactionId,MatrixUtil.DEBUG_LOG);
		ArrayList attachmentList = (ArrayList)attachments.get(transactionId);
		if(attachmentList!=null)
		{
			attachmentList.add(attachment);			
		}
		else
		{
			attachmentList = new ArrayList();
			attachmentList.add(attachment);
		}
		attachments.put(transactionId,attachmentList);
	}
	
	//	Added by Reena on 14 May 2003 - Attachments
	/**
	 * Retrieve Response Attachments for a transaction
	 * @param transactionId Transaction ID
	 * @return List of Attachments associated with the transaction 
	 */
	public ArrayList getAttachmentList(String transactionId)
	{
		MatrixUtil.toLog(transactionId,"SRBStore - getAttachmentList()-BEGIN",MatrixUtil.DEBUG_LOG);
		MatrixUtil.toLog(transactionId,"SRBStore - getAttachmentList() -transactionID:"+transactionId,MatrixUtil.DEBUG_LOG);
		MatrixUtil.toLog(transactionId,"SRBStore - getAttachmentList() - entered getAttachmentList",MatrixUtil.DEBUG_LOG);
	    ArrayList attachmentList = (ArrayList)attachments.get(transactionId);
		MatrixUtil.toLog(transactionId,"SRBStore - getAttachmentList() - pulled out of hashtable",MatrixUtil.DEBUG_LOG);
		MatrixUtil.toLog(transactionId,"SRBStore - getAttachmentList()-END",MatrixUtil.DEBUG_LOG);
	    return(attachmentList);
	}
	
	

    // example usage
    /*
    public static void main(String[] args){

	SRBStore srbStore = new SRBStore();
	String[] steps = {"Step 1", "Step 2", "Step 3"};
	srbStore.setTransactionStepnames("Transaction 1", "evk", steps);
	srbStore.setStepStatus("Transaction 1", "Step 1", new StepStatusInfo("Step 1", "Finished", 0));

	String[] stepnames = srbStore.getTransactionStepnames("Transaction 1");

	for(int i=0; i<stepnames.length; i++){
	    StepStatusInfo status = srbStore.getStepStatus("Transaction 1", stepnames[i]);
	    System.out.println("Transaction 1, Step " + i + ": " + status);
	}
    }
    */
    
	//	Added by Reena on 19 June 2003
	/**
	 * Set the Status for a flow
	 * @param txnId Transaction iD
	 * @param flowId Flow ID
	 * @param flowStatusResponse Flow Status Response
	 */
	public void setMatrixFlowStatus(String txnId,String flowId,Object flowStatusResponse)
	{
		MatrixUtil.toLog(txnId,"SRBStore - setMatrixFlowStatus() -txnId-flowId:"+txnId+"."+flowId,MatrixUtil.DEBUG_LOG);
		FlowStatusResponse response =(FlowStatusResponse) flowStatusResponse;
		MatrixUtil.toLog(txnId,"SRBStore - setTransactionStatus() -FLOW RESPONSE STATUS:"+response.getStatusCode(),MatrixUtil.DEBUG_LOG);
		matrixFlow.put(createkey(txnId,flowId),flowStatusResponse);
	}
    
    /**
     * Retrieve the status for a flow
     * @param txnId  Transaction ID
     * @param flowId Flow ID
     * @return Flow Status Response
     */
	public Object getMatrixFlowStatus(String txnId,String flowId)
	{
		MatrixUtil.toLog(txnId,"SRBStore - getMatrixFlowStatus() -txnId-flowId:"+txnId+"."+flowId,MatrixUtil.DEBUG_LOG);
		if(matrixFlow.get(createkey(txnId,flowId))!=null)
		{
			FlowStatusResponse response =(FlowStatusResponse) matrixFlow.get(createkey(txnId,flowId));
			MatrixUtil.toLog(txnId,"SRBStore - getMatrixFlowStatus() -FLOW RESPONSE STATUS:"+response.getStatusCode(),MatrixUtil.DEBUG_LOG);
		}	
		return(matrixFlow.get(createkey(txnId,flowId)));
	}
	
	//	Added by Reena on 19 June 2003
	/**
	 * Set the status for a transaction
	 * @param txnId Transaction ID
	 * @param transactionStatusResponse Transaction Status Response
	 */
	public void setTransactionStatus(String txnId,Object transactionStatusResponse)
	{
		MatrixUtil.toLog(txnId,"SRBStore - setTransactionStatus() -txnId:"+txnId,MatrixUtil.DEBUG_LOG);		
		TransactionStatusResponse response =(TransactionStatusResponse) transactionStatusResponse;
		MatrixUtil.toLog(txnId,"SRBStore - setTransactionStatus() -TXN RESPONSE STATUS:"+response.getStatusCode(),MatrixUtil.DEBUG_LOG);
		transactionStatus.put(txnId,transactionStatusResponse);
	}
    
    /**
     * Retrieve the Status for a transaction
     * @param txnId Transaction Id
     * @return Transaction Status Response
     */
	public Object getTransactionStatus(String txnId)
	{
		MatrixUtil.toLog(txnId,"SRBStore - getTransactionStatus() -txnId:"+txnId,MatrixUtil.DEBUG_LOG);
		if(transactionStatus.get(txnId)!=null)
		{
			TransactionStatusResponse response =(TransactionStatusResponse) transactionStatus.get(txnId);
			MatrixUtil.toLog(txnId,"SRBStore - getTransactionStatus() -TXN RESPONSE STATUS:"+response.getStatusCode(),MatrixUtil.DEBUG_LOG);
		}	
		return(transactionStatus.get(txnId));
	}
	
	// Added by Reena Mathew on 18 August 2003 
	/**
	 * Retrieve a request attachment for a specific transaction id and content id
	 * @param txnID Transaction ID
	 * @param contentID Content ID
	 * @return Request Attachment for a transaction with a specific content id
	 */
	public Object getInputAttachment(String txnID,String contentID)
	{
		MatrixUtil.toLog(txnID,"SRBStore - getInputAttachment()-BEGIN" ,MatrixUtil.DEBUG_LOG);	
		MatrixUtil.toLog(txnID,"SRBStore - getInputAttachment() -txnID.contentID:"+txnID+"."+contentID,MatrixUtil.DEBUG_LOG);
		MatrixUtil.toLog(txnID,"SRBStore - getInputAttachment() -END",MatrixUtil.DEBUG_LOG);
		return(transactionInputAttachments.get(createkey(txnID,contentID)));	
	}
	
	/**
	 * Save Request Attachment
	 * @param txnID Transaction ID
	 * @param contentID Content ID
	 * @param attachment Attachment to be saved
	 */
	public void addInputAttachment(String txnID,String contentID,Object attachment)
	{
		transactionInputAttachments.put(createkey(txnID,contentID),attachment);
	}
	
	/**
	 * Set the Request Input Attachments
	 * @param inputAttachments Request Attachments
	 */
	public void setInputAttachmentForTransaction(Hashtable inputAttachments)
	{
		transactionInputAttachments = inputAttachments;
	}
	
	/**
	 * Retrieve Request Attachments
	 * @return Request Input Attachments
	 */
	public Hashtable getInputAttachmentForTransaction()
	{
			return(transactionInputAttachments);
	}
	
}
