/**************************************************************************
Copyright Notice
All Rights Reserved
Please refer to files in COPYRIGHT directory under the SRB software directory 
for copyright, usage, liability and license information.
Please read these files before using,modifying or distributing SRB software.
**************************************************************************/
 


/* Slscont.c - Display containers and contents 
 */

#include "scommands.h"

 

extern char srbAuth[];
extern char srbHost[];
extern char srbUser[];
extern char defaultResource[];
extern char mdasCollectionName[];
extern char inCondition[];

 
void
usage(char *prog)
{
    fprintf(stderr,"Usage: %s [-a] [-l] [-z mcatName] [container]\n", prog);
}

int
main(int argc, char **argv)
{
    srbConn *conn;
    char resourceName[MAX_TOKEN], dataType[MAX_TOKEN];
    int contSize;
    int i;
    int aFlag = 0;
    int lFlag = 0;
    int contInput;
    int c;
    char *containerName;
    mdasC_sql_result_struct  myresult;
    char *userName, *domainName;
    char contName[MAX_TOKEN], contCollection[MAX_TOKEN],
     fullContName[MAX_TOKEN];
    int status;
    int flagval = 0;
    char *mcatName = NULL;
    char myMcatName[MAX_TOKEN];

    strcpy(inCondition , ""); 
    while ((c=getopt(argc, argv, "alz:")) != EOF) {
        switch (c) {
            case 'a':
		aFlag = 1;
		flagval |= a_FLAG;
		break;
            case 'l':
		lFlag = 1;
		flagval |= l_FLAG;
		break;
            case 'z':
                strcpy (myMcatName, optarg);
                flagval |= z_FLAG;
                break;
            default:
                usage (argv[0]);
                exit (1);
        }
    }

    i = initSrbClientEnv();
    if (i < 0){
	printf("Slscont: Error:%i\n",i); 
	exit(1);
    }

    conn = srbConnect (srbHost, NULL, srbAuth,
     NULL, NULL, NULL, NULL);
    if (clStatus(conn) != CLI_CONNECTION_OK) {
        fprintf(stderr,"Slscont: Connection to srbMaster failed.\n");
        fprintf(stderr,"%s",clErrorMessage(conn));
	srb_perror (2, clStatus(conn), "", SRB_RCMD_ACTION|SRB_LONG_MSG);
	clFinish(conn); exit(3);
    }

    if (conn->clientUser == NULL) {
	userName = conn->proxyUser->userName;
	domainName = conn->proxyUser->domainName;
    } else {
	userName = conn->clientUser->userName;
	domainName = conn->clientUser->domainName;
    }

    /* list container metadata */
    /* Changes made by Roman for Wildcards and Output format */
    if (optind >= argc-1 || lFlag == 1) {
	if (optind >= argc) {
#ifdef FED_MCAT
            if ((flagval & z_FLAG) == 0) {
                char *thisMcat = NULL;

                thisMcat = getmcatNamefromHint (mdasCollectionName);
                if (thisMcat == NULL) {
                    status =srbGetMcatZone (conn, userName, domainName,
                     myMcatName);
                    if (status >= 0)
                        mcatName = myMcatName;
                    else
                        mcatName = NULL;
                } else if (strcmp (thisMcat, "home") == 0 ||
                 strcmp (thisMcat, "container") == 0) {
                    mcatName = NULL;
                } else {
                    mcatName = thisMcat;
                }
            } else {
                mcatName = myMcatName;
            }
#else
            mcatName = NULL;
#endif

	    status = lsContainer (conn, NULL, userName, domainName, flagval, 
	     &myresult, mcatName);
            if(status == 0)   /* 9-7-00 to pass upcoming if statement */       
               status = 1;
	}
        else {
            containerName = argv[optind];                /* Roman 9-6-00 begin */

#ifdef FED_MCAT
            if (containerName[0] != '/') {    /* full path */
    		if ((flagval & z_FLAG) == 0) {
                    char *thisMcat = NULL;

                    thisMcat = getmcatNamefromHint (mdasCollectionName);
                    if (thisMcat == NULL) {
                        status =srbGetMcatZone (conn, userName, domainName,
                         myMcatName);
                        if (status >= 0)
                            mcatName = myMcatName;
                        else
                            mcatName = NULL;
                    } else if (strcmp (thisMcat, "home") == 0 ||
                     strcmp (thisMcat, "container") == 0) {
                        mcatName = NULL;
                    } else {
                        mcatName = thisMcat;
                    }
   	         } else {
        	    mcatName = myMcatName;
    		 }
             } else {
                 mcatName = NULL;
             }
#else
            mcatName = NULL;
#endif
            parseContainerName (containerName, contName, contCollection,
             userName, domainName, mcatName);
                
            sprintf (fullContName, "%s/%s", contCollection, contName);
	    status = lsContainer (conn, fullContName, userName, domainName, 
	     flagval, &myresult, mcatName);         /* Roman 9-6-00 end   */
	}

        if (status != 0 || lFlag == 1) {         /* Roman 9-7-00 if */
           printf ("Containers:\n");
    	   if (status < 0) {
	       if (status != CLI_NO_ANSWER && status != MCAT_INQUIRE_ERROR) {
	           fprintf (stderr,
		    "Slscont: Error listing container 'status =  %i\n", status);
	    	   srb_perror (2, i, "", SRB_RCMD_ACTION|SRB_LONG_MSG);
		   clFinish(conn); exit(4);  /* Raja */
	       }
	       clFinish(conn);   /* Raja */
  	       exit(0);          /* Raja */
    	   }
	   else if (myresult.result_count == 0) {
  	       clFinish(conn);
  	       exit(0);
    	   }

    	   if (lFlag > 0) {
	       printf ("  %-70s  %-15s  %-10s  %-10s  %-7s\n", 
	        "Container Name", "Resource", "Size", "MaxSize", "Count"); /* Roman 9-6-00 */
    	       prtContainerLong (&myresult);
    	   } 
           else {
    	       prtContainer (&myresult);
           }

    	   while (myresult.continuation_index >= 0) {
               clearSqlResult (&myresult);
               status = srbGetMoreRows(conn, MDAS_CATALOG,
                 myresult.continuation_index, &myresult, MAX_GET_RESULTS);
               if (myresult.result_count == 0  || status != 0)
            	   break;
    	       prtContainer (&myresult);
    	   }
           clearSqlResult (&myresult);
           clFinish(conn);
           exit(0);
        }                                      /* Roman 9-7-00 end if */
    } 

    if (optind < argc && lFlag != 1) {         /* Roman 9-6-00 */
        printf ("Containers:\n");              /* Roman 9-5-00 */
	for (i = optind; i < argc; i++) {
	    containerName = argv[i];

#ifdef FED_MCAT
            if ((i == optind || (i > optind && mcatName == NULL)) && 
	     containerName[0] != '/') {
                if ((flagval & z_FLAG) == 0) {
                    char *thisMcat = NULL;

                    thisMcat = getmcatNamefromHint (mdasCollectionName);
                    if (thisMcat == NULL) {
                        status =srbGetMcatZone (conn, userName, domainName,
                         myMcatName);
                        if (status >= 0)
                            mcatName = myMcatName;
                        else
                            mcatName = NULL;
                    } else if (strcmp (thisMcat, "home") == 0 ||
                     strcmp (thisMcat, "container") == 0) {
                        mcatName = NULL;
                    } else {
                        mcatName = thisMcat;
                    }
                } else {
                    mcatName = myMcatName;
                }

            } else {
                mcatName = NULL;
            }
#else
            mcatName = NULL;
#endif

    	    parseContainerName (containerName, contName, contCollection,
     	     userName, domainName, mcatName);

	    sprintf (fullContName, "%s/%s", contCollection, contName);

	    status = lsInContainer (conn, userName, domainName, fullContName,
	     flagval, &myresult);

            if (status != 1) {                 /* Roman 9-7-00 if */
               if (status < 0) {
            	   if (status != CLI_NO_ANSWER && status != MCAT_INQUIRE_ERROR) {
                       fprintf (stderr,
                        "Slscont: Error listing container 'status =  %i\n", status);
                       srb_perror (2, i, "", SRB_RCMD_ACTION|SRB_LONG_MSG);
		       clFinish(conn); exit(5);  /* Roman 9-5-00 */
            	   }
            	   clFinish(conn);   /* Roman 9-5-00 */
                   exit(0);          /* Roman 9-5-00 */
               }
               else if (myresult.result_count == 0) {
            	   clFinish(conn);
            	   exit(0);
               }

               printf ("%s\n", fullContName);     /* Roman 9-5-00 */
               printf ("  %-100s  %-10s  %-10s\n",
                "InContainer Objects", "Offset", "Size");   /* Roman 9-5-00 */

	       prtInContainer (&myresult, fullContName);

               while (myresult.continuation_index >= 0) {
            	   clearSqlResult (&myresult);
            	   status = srbGetMoreRows(conn, MDAS_CATALOG,
                    myresult.continuation_index, &myresult, MAX_GET_RESULTS);
            	   if (myresult.result_count == 0  || status != 0)
                       break;
            	   prtInContainer (&myresult, fullContName);
               }
               clearSqlResult (&myresult);
            }
	}
    }
    clFinish(conn);
    exit(0);
}

int
prtContainer (mdasC_sql_result_struct *myresult)
{
    int i;
    char *containerName;


    containerName = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[CONTAINER_NAME], dcs_aname[CONTAINER_NAME]);

    for (i = 0; i < myresult->row_count; i++) {
	printf ("  %-s\n", containerName);
	containerName += MAX_DATA_SIZE;
    }
    return (0);
}

int
prtContainerLong (mdasC_sql_result_struct *myresult)
{
    int i;
    char *containerName, *maxContainerSize, *size, *resource, *count;
    int iCount;


    containerName = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[CONTAINER_NAME], dcs_aname[CONTAINER_NAME]);
    maxContainerSize = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[CONTAINER_MAX_SIZE], dcs_aname[CONTAINER_MAX_SIZE]);
    size = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[CONTAINER_SIZE], dcs_aname[CONTAINER_SIZE]);
    resource = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[PHY_RSRC_NAME], dcs_aname[PHY_RSRC_NAME]);
    count = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[DATA_ID], dcs_aname[DATA_ID]);

    for (i = 0; i < myresult->row_count; i++) {
	iCount = atoi (count) - 1;
	printf ("  %-70s  %-15s  %-10s  %-10s  %-7d\n", 
	 containerName, resource, size, maxContainerSize, iCount);
	containerName += MAX_DATA_SIZE;
	resource += MAX_DATA_SIZE;
	size += MAX_DATA_SIZE;
	maxContainerSize += MAX_DATA_SIZE;
	count += MAX_DATA_SIZE;
    }
    return (0);
}

int
prtInContainer (mdasC_sql_result_struct *myresult, char *containerName)
{

/* 9-12-00 Roman, removed *containerName bug */
/*    char *containerName, *dataName, *collection, *size, *offset;  Roman  */

    int i;
    char *dataName, *collection, *size, *offset;
    char fullName[MAX_TOKEN];

    dataName = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[DATA_NAME], dcs_aname[DATA_NAME]);
    if (dataName == NULL)
       return (0);
    collection = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[DATA_GRP_NAME], dcs_aname[DATA_GRP_NAME]);
    size = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[SIZE], dcs_aname[SIZE]);
    offset = (char *) getFromResultStruct(
      (mdasC_sql_result_struct *) myresult,
        dcs_tname[OFFSET], dcs_aname[OFFSET]);

    for (i = 0; i < myresult->row_count; i++) {
	sprintf (fullName, "%s/%s", collection, dataName);

        if (strcmp (containerName, fullName) != 0)  /* skip container */
            printf ("  %-100s  %-10s  %-10s\n", fullName, offset, size);

	collection += MAX_DATA_SIZE;
	dataName += MAX_DATA_SIZE;
	size += MAX_DATA_SIZE;
	offset += MAX_DATA_SIZE;
    }
    return (0);
}


